.autocomplete_light_widget .autocomplete {margin-bottom:5px;}
.autocomplete_light_widget .yourlabs_autocomplete { border: none; background: none; }
.autocomplete_light_widget .yourlabs_autocomplete.outer_container { position: relative; }
.autocomplete_light_widget .yourlabs_autocomplete.inner_container { position: absolute; left: -1px; top: 2px; border: 1px solid #E0EDF5; width: 260px; background: #fff; }
.autocomplete_light_widget .yourlabs_autocomplete ul { margin: 0; padding: 0; overflow-x: hidden; overflow-y: auto; position: relative; list-style-type: none; max-height: 200px; }
.autocomplete_light_widget .yourlabs_autocomplete li { cursor: pointer; color: #000; padding: 5px 6px; }
.autocomplete_light_widget .yourlabs_autocomplete li.active { color: #fff; background: #2e83db; }
.autocomplete_light_widget .deck { padding-left: 165px; width: 262px; }
.autocomplete_light_widget .deck li { font-size: 10px; margin-bottom: 5px; padding: 5px; background: -moz-linear-gradient(center top , #F4F7F9 0%, #EBF1F5 100%) repeat scroll 0 0 transparent; border-radius: 5px; list-style-type: none; }
.autocomplete_light_widget .deck li span { display: inline-block; width: 210px; }
.autocomplete_light_widget .deck li .remove { float: left; margin-right: 10px; color: #fff; text-indent: -999px; height: 15px; width: 15px; background: url('../admin/img/inline-delete.png'), url('../admin/img/admin/inline-delete.png'); margin-top: 2px; cursor: pointer; }
.autocomplete_light_widget .deck li .remove:hover { background-position: -15px 0; }
.autocomplete_light_widget .deck li:last-child { margin-bottom: 0; }
